@@ -20,6 +20,7 @@ from typing import List, Optional, Set
 
 from kfp_tekton.compiler._k8s_helper import sanitize_k8s_name
 from kfp_tekton.compiler._op_to_template import _get_base_step, _add_mount_path, _prepend_steps
+from os import environ as env
 
 
 def fix_big_data_passing(workflow: dict) -> dict:
@@ -300,19 +301,10 @@ def fix_big_data_passing(workflow: dict) -> dict:
     # Use workspaces to tasks if big data passing instead of 'results', 'copy-inputs'
     for task_template in container_templates:
         task_template = big_data_passing_tasks(task_template,
+                                               pipelinerun_template,
                                                inputs_consumed_as_artifacts,
                                                outputs_consumed_as_artifacts)
 
-    # # Create pvc for pipelinerun if big data passing.
-    # # As we used workspaces in tekton pipelines which depends on it.
-    # # User need to create PV manually, or enable dynamic volume provisioning, refer to the link of:
-    # # https://kubernetes.io/docs/concepts/storage/dynamic-provisioning
-    # # TODO: Remove PVC, use 'volumeClaimTemplate' instead, issue #181.
-    # User need to create pvc manually recently until issue #181 addressed
-    # if pipelinerun_workspaces:
-    #     for pipelinerun in pipelinerun_workspaces:
-    #         workflow.append(create_pvc(pipelinerun))
-
     # Remove input parameters unless they're used downstream.
     # This also removes unused container template inputs if any.
     for template in container_templates + [pipeline_template]:
@@ -453,18 +445,29 @@ def big_data_passing_pipelinerun(name: str, pr: dict, pw: set):
     pipelinerun_name = name
     if pipelinerun_name in pw:
         pr.get('spec', {}).setdefault('workspaces', [])
+        # Change persistentVolumeClaim to volumeClaimTemplate which need Tekton version > 0.12
+        # User could modify default size of storage in the yaml file mannually if necessary.
+        DEFAULT_ACCESSMODES = env.get('DEFAULT_ACCESSMODES', 'ReadWriteMany')
+        DEFAULT_STORAGE_SIZE = env.get('DEFAULT_STORAGE_SIZE', '2Gi')
         pr['spec']['workspaces'].append({
             "name": pipelinerun_name,
-            "persistentVolumeClaim": {
-                "claimName": pipelinerun_name
+            "volumeClaimTemplate": {
+                "spec": {
+                    "accessModes": [DEFAULT_ACCESSMODES],
+                    "resources": {
+                        "requests": {
+                            "storage": DEFAULT_STORAGE_SIZE
+                        }
+                    }
+                }
             }
         })
         prw.add(pipelinerun_name)
     return pr, prw
 
 
-def big_data_passing_tasks(task: dict, inputs_tasks: set,
-                           outputs_tasks: set) -> dict:
+def big_data_passing_tasks(task: dict, pipelinerun_template: dict,
+                           inputs_tasks: set, outputs_tasks: set) -> dict:
     task_name = task.get('name')
     task_spec = task.get('taskSpec', {})
     # Data passing for the task outputs
@@ -482,6 +485,8 @@ def big_data_passing_tasks(task: dict, inputs_tasks: set,
                 task_name, task_name, task_output.get('name'))
             task['taskSpec'] = replace_big_data_placeholder(
                 task['taskSpec'], placeholder, workspaces_parameter)
+            pipelinerun_template['metadata']['annotations'] = replace_big_data_placeholder(
+                pipelinerun_template['metadata']['annotations'], placeholder, workspaces_parameter)
 
     # Remove artifacts outputs from results
     task['taskSpec']['results'] = [
@@ -527,31 +532,6 @@ def big_data_passing_tasks(task: dict, inputs_tasks: set,
     return task
 
 
-# Create pvc for pipelinerun if using big data passing.
-# As we used workspaces in tekton pipelines which depends on it.
-# User need to create PV manually, or enable dynamic volume provisioning, refer to the link of:
-# https://kubernetes.io/docs/concepts/storage/dynamic-provisioning
-# TODO: Remove PVC if Tekton version > = 0.12, use 'volumeClaimTemplate' instead
-def create_pvc(pr: str) -> dict:
-    pvc = {
-        "apiVersion": "v1",
-        "kind": "PersistentVolumeClaim",
-        "metadata": {
-            "name": pr
-        },
-        "spec": {
-            "accessModes": ["ReadWriteOnce"],
-            "resources": {
-                "requests": {
-                    "storage": "100Mi"
-                }
-            },
-            "volumeMode": "Filesystem",
-        }
-    }
-    return pvc
-
-
 def input_artifacts_tasks(template: dict, artifact: dict) -> dict:
     # The input artifacts in KFP is not pulling from s3, it will always be passed as a raw input.
     # Visit https://github.com/kubeflow/pipelines/issues/336 for more details on the implementation.
