@@ -14,7 +14,6 @@
 
 import json
 import re
-import textwrap
 import yaml
 
 from collections import OrderedDict
@@ -101,7 +100,8 @@ def _get_resourceOp_template(op: BaseOp,
                     "type": "string"
                 },
                 {
-                    # Todo: The image need to be replaced, once there are official images from tekton
+                    # This image is hosted by the kfp-tekton maintainers
+                    # Source code: https://github.com/kubeflow/kfp-tekton/tree/master/tekton-catalog/kubectl-wrapper
                     "default": "index.docker.io/aipipeline/kubeclient:v0.0.2",
                     "description": "Kubectl wrapper image",
                     "name": "image",
@@ -248,7 +248,8 @@ def _process_parameters(processed_op: BaseOp,
                     s['args'] = args
             # If file output path cannot be found/replaced, use emptyDir to copy it to the tekton/results path
             if need_copy_step:
-                copy_results_step['script'] = copy_results_step['script'] + 'cp ' + path + ' $(results.%s.path);' % name + '\n'
+                copy_results_step['script'] = copy_results_step['script'] + 'cp ' + path + ' $(results.%s.path);' \
+                                                % sanitize_k8s_name(name) + '\n'
                 mount_path = path.rsplit("/", 1)[0]
                 if mount_path not in mounted_param_paths:
                     _add_mount_path(name, path, mount_path, volume_mount_step_template, volume_template, mounted_param_paths)
@@ -265,19 +266,12 @@ def _process_output_artifacts(outputs_dict: Dict[Text, Any],
                               volume_mount_step_template: List[Dict[Text, Any]],
                               volume_template: List[Dict[Text, Any]],
                               replaced_param_list: List[Text],
-                              artifact_to_result_mapping: Dict[Text, Any]):
-    """Process output artifacts to replicate the same behavior as Argo.
+                              artifact_to_result_mapping: Dict[Text, Any],
+                              artifact_items: List[Any]):
+    """Process output artifact dependencies to replicate the same behavior as Argo.
 
-    For storing artifacts, we will be using the minio/mc image because we need to upload artifacts to any type of
-    object storage and endpoint. The minio/mc is the best image suited for this task because the default KFP
-    is using minio and it also works well with other s3/gcs type of storage.
-
-    - image: minio/mc
-        name: copy-artifacts
-        script: |
-            #!/usr/bin/env sh
-            mc config host add storage http://minio-service.$NAMESPACE:9000 $AWS_ACCESS_KEY_ID $AWS_SECRET_ACCESS_KEY
-            mc cp /tmp/file.txt storage/$(inputs.params.bucket)/runs/$PIPELINERUN/$TASKRUN/file.txt
+    For storing artifacts, we will need to provide the output artifact dependencies for the server to
+    find and store the artifacts with the proper metadata.
 
     Args:
         outputs_dict {Dict[Text, Any]}: Dictionary of the possible parameters/artifacts in this task
@@ -289,54 +283,23 @@ def _process_output_artifacts(outputs_dict: Dict[Text, Any],
     Returns:
         Dict[Text, Any]
     """
+    
     if outputs_dict.get('artifacts'):
-        # TODO: Pull default values from KFP configmap when integrated with KFP.
-        storage_location = outputs_dict['artifacts'][0].get('s3', {})
-        insecure = storage_location.get("insecure", True)
-        endpoint = storage_location.get("endpoint", "minio-service.$NAMESPACE:9000")
-        # We want to use the insecure flag to figure out whether to use http or https scheme
-        endpoint = re.sub(r"https?://", "", endpoint)
-        endpoint = 'http://' + endpoint if insecure else 'https://' + endpoint
-        access_key = storage_location.get("accessKeySecret", {"name": "mlpipeline-minio-artifact", "key": "accesskey"})
-        secret_access_key = storage_location.get("secretKeySecret", {"name": "mlpipeline-minio-artifact", "key": "secretkey"})
-        bucket = storage_location.get("bucket", "mlpipeline")
-        copy_artifacts_step = {
-            'image': 'minio/mc',
-            'name': 'copy-artifacts',
-            'script': textwrap.dedent('''\
-                        #!/usr/bin/env sh
-                        mc config host add storage %s $AWS_ACCESS_KEY_ID $AWS_SECRET_ACCESS_KEY
-                        ''' % (endpoint)),
-            'env': [
-                {'name': 'PIPELINERUN', 'valueFrom': {'fieldRef': {'fieldPath': "metadata.labels['tekton.dev/pipelineRun']"}}},
-                {'name': 'PIPELINETASK', 'valueFrom': {'fieldRef': {'fieldPath': "metadata.labels['tekton.dev/pipelineTask']"}}},
-                {'name': 'NAMESPACE', 'valueFrom': {'fieldRef': {'fieldPath': "metadata.namespace"}}},
-                {'name': 'AWS_ACCESS_KEY_ID', 'valueFrom': {'secretKeyRef': {'name': access_key['name'], 'key': access_key['key']}}},
-                {'name': 'AWS_SECRET_ACCESS_KEY', 'valueFrom': {'secretKeyRef': {'name': secret_access_key['name'],
-                                                                                 'key': secret_access_key['key']}}}
-            ]
-        }
         mounted_artifact_paths = []
         for artifact in outputs_dict['artifacts']:
             artifact_name = artifact_to_result_mapping.get(artifact['name'], artifact['name'])
             if artifact['name'] in replaced_param_list:
-                copy_artifacts_step['script'] = copy_artifacts_step['script'] + \
-                    'tar -cvzf %s.tgz $(results.%s.path)\n' % (artifact_name, sanitize_k8s_name(artifact_name)) + \
-                    'mc cp %s.tgz storage/%s/artifacts/$PIPELINERUN/$PIPELINETASK/%s.tgz\n' % (artifact_name,
-                                                                                     bucket, artifact_name)
+                artifact_items.append([artifact_name, "$(results.%s.path)" % sanitize_k8s_name(artifact_name)])
             else:
-                copy_artifacts_step['script'] = copy_artifacts_step['script'] + \
-                    'tar -cvzf %s.tgz %s\n' % (artifact_name, artifact['path']) + \
-                    'mc cp %s.tgz storage/%s/artifacts/$PIPELINERUN/$PIPELINETASK/%s.tgz\n' % (artifact_name, bucket, artifact_name)
+                artifact_items.append([artifact_name, artifact['path']])
                 if artifact['path'].rsplit("/", 1)[0] not in mounted_artifact_paths:
+                    if artifact['path'].rsplit("/", 1)[0] == "":
+                        raise ValueError('Undefined volume path or "/" path artifacts are not allowed.')
                     volume_mount_step_template.append({
                         'name': sanitize_k8s_name(artifact['name']), 'mountPath': artifact['path'].rsplit("/", 1)[0]
                     })
                     volume_template.append({'name': sanitize_k8s_name(artifact['name']), 'emptyDir': {}})
                     mounted_artifact_paths.append(artifact['path'].rsplit("/", 1)[0])
-        return copy_artifacts_step
-    else:
-        return {}
 
 
 def _process_base_ops(op: BaseOp):
@@ -369,7 +332,9 @@ def _process_base_ops(op: BaseOp):
     return op
 
 
-def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifacts=False):
+def _op_to_template(op: BaseOp,
+                    pipelinerun_output_artifacts={},
+                    artifact_items={}):
     """Generate template given an operator inherited from BaseOp."""
 
     # initial local variables for tracking volumes and artifacts
@@ -396,7 +361,7 @@ def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifact
         output_artifacts = [
             {'name': name, 'path': path}
             for name, path in output_artifact_paths.items()
-        ] if enable_artifacts else []
+        ]
 
         # workflow template
         container = convert_k8s_obj_to_json(
@@ -417,16 +382,17 @@ def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifact
         }
 
         # Create output artifact tracking annotation.
-        if enable_artifacts:
-            for output_artifact in output_artifacts:
-                output_annotation = pipelinerun_output_artifacts.get(processed_op.name, [])
-                output_annotation.append(
-                    {
-                        'name': output_artifact['name'],
-                        'path': output_artifact['path']
-                    }
-                )
-                pipelinerun_output_artifacts[processed_op.name] = output_annotation
+        for output_artifact in output_artifacts:
+            output_annotation = pipelinerun_output_artifacts.get(processed_op.name, [])
+            output_annotation.append(
+                {
+                    'name': output_artifact.get('name', ''),
+                    'path': output_artifact.get('path', ''),
+                    'key': "artifacts/$PIPELINERUN/%s/%s.tgz" %
+                    (processed_op.name, output_artifact.get('name', '').replace(processed_op.name + '-', ''))
+                }
+            )
+            pipelinerun_output_artifacts[processed_op.name] = output_annotation
 
     elif isinstance(op, dsl.ResourceOp):
         # no output artifacts
@@ -470,6 +436,7 @@ def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifact
         op_outputs = {}
         param_outputs = {}
     outputs_dict = _outputs_to_json(op, op_outputs, param_outputs, output_artifacts)
+    artifact_items[op.name] = artifact_items.get(op.name, [])
     if outputs_dict:
         copy_results_step = _process_parameters(processed_op,
                                                 template,
@@ -479,16 +446,15 @@ def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifact
                                                 replaced_param_list,
                                                 artifact_to_result_mapping,
                                                 mounted_param_paths)
-        copy_artifacts_step = _process_output_artifacts(outputs_dict,
-                                                        volume_mount_step_template,
-                                                        volume_template,
-                                                        replaced_param_list,
-                                                        artifact_to_result_mapping)
+        _process_output_artifacts(outputs_dict,
+                                  volume_mount_step_template,
+                                  volume_template,
+                                  replaced_param_list,
+                                  artifact_to_result_mapping,
+                                  artifact_items[op.name])
         if mounted_param_paths:
             template['spec']['steps'].append(copy_results_step)
         _update_volumes(template, volume_mount_step_template, volume_template)
-        if copy_artifacts_step:
-            template['spec']['steps'].append(copy_artifacts_step)
 
     # metadata
     if processed_op.pod_annotations or processed_op.pod_labels:
@@ -514,7 +480,7 @@ def _op_to_template(op: BaseOp, pipelinerun_output_artifacts={}, enable_artifact
 
     # volumes
     if processed_op.volumes:
-        template['spec']['volumes'] = template['spec'].get('volume', []) + [convert_k8s_obj_to_json(volume)
+        template['spec']['volumes'] = template['spec'].get('volumes', []) + [convert_k8s_obj_to_json(volume)
                                                                             for volume in processed_op.volumes]
         template['spec']['volumes'].sort(key=lambda x: x['name'])
 
