@@ -23,10 +23,12 @@ import re
 import textwrap
 
 from typing import Callable, List, Text, Dict, Any
+from os import environ as env
+from distutils.util import strtobool
 
 # Kubeflow Pipeline imports
 from kfp import dsl
-from kfp.compiler._default_transformers import add_pod_env, add_pod_labels, get_default_telemetry_labels
+from kfp.compiler._default_transformers import add_pod_env  # , add_pod_labels, get_default_telemetry_labels
 from kfp.compiler.compiler import Compiler
 # from kfp.components._yaml_utils import dump_yaml
 from kfp.components.structures import InputSpec
@@ -36,10 +38,16 @@ from kfp.dsl._metadata import _extract_pipeline_metadata
 # KFP-Tekton imports
 from kfp_tekton.compiler import __tekton_api_version__ as tekton_api_version
 from kfp_tekton.compiler._data_passing_rewriter import fix_big_data_passing
-from kfp_tekton.compiler._k8s_helper import convert_k8s_obj_to_json, sanitize_k8s_name
+from kfp_tekton.compiler._k8s_helper import convert_k8s_obj_to_json, sanitize_k8s_name, sanitize_k8s_object
 from kfp_tekton.compiler._op_to_template import _op_to_template
 
 
+DEFAULT_ARTIFACT_BUCKET = env.get('DEFAULT_ARTIFACT_BUCKET', 'mlpipeline')
+DEFAULT_ARTIFACT_ENDPOINT = env.get('DEFAULT_ARTIFACT_ENDPOINT', 'minio-service.kubeflow:9000')
+DEFAULT_ARTIFACT_ENDPOINT_SCHEME = env.get('DEFAULT_ARTIFACT_ENDPOINT_SCHEME', 'http://')
+TEKTON_GLOBAL_DEFAULT_TIMEOUT = strtobool(env.get('TEKTON_GLOBAL_DEFAULT_TIMEOUT', 'false'))
+
+
 def _get_super_condition_template():
 
   python_script = textwrap.dedent('''\
@@ -93,14 +101,11 @@ class TektonCompiler(Compiler):
   """
 
   def __init__(self, **kwargs):
-    # Intentionally set self.enable_artifacts to None because when _create_pipeline_workflow
-    # is called directly (e.g. in the case of there being no pipeline decorator), then
-    # self.enable_artifacts is not set
-    #
     # Input and output artifacts are hash maps for metadata tracking.
-    self.enable_artifacts = None
+    # artifact_items is the artifact dependency map
     self.input_artifacts = {}
     self.output_artifacts = {}
+    self.artifact_items = {}
     super().__init__(**kwargs)
 
   def _get_loop_task(self, task: Dict, op_name_to_for_loop_op):
@@ -222,7 +227,9 @@ class TektonCompiler(Compiler):
       op_to_templates_handler: Handler which converts a base op into a list of argo templates.
     """
 
-    op_to_steps_handler = op_to_templates_handler or (lambda op: [_op_to_template(op, self.output_artifacts, self.enable_artifacts)])
+    op_to_steps_handler = op_to_templates_handler or (lambda op: [_op_to_template(op,
+                                                                  self.output_artifacts,
+                                                                  self.artifact_items)])
     root_group = pipeline.groups[0]
 
     # Call the transformation functions before determining the inputs/outputs, otherwise
@@ -379,13 +386,13 @@ class TektonCompiler(Compiler):
         # Process input parameters if needed
         if isinstance(condition.operand1, dsl.PipelineParam):
           if condition.operand1.op_name:
-            operand_value = '$(tasks.' + condition.operand1.op_name + '.results.' + condition.operand1.name + ')'
+            operand_value = '$(tasks.' + condition.operand1.op_name + '.results.' + sanitize_k8s_name(condition.operand1.name) + ')'
           else:
             operand_value = '$(params.' + condition.operand1.name + ')'
           input_params.append(operand_value)
         if isinstance(condition.operand2, dsl.PipelineParam):
           if condition.operand2.op_name:
-            operand_value = '$(tasks.' + condition.operand2.op_name + '.results.' + condition.operand2.name + ')'
+            operand_value = '$(tasks.' + condition.operand2.op_name + '.results.' + sanitize_k8s_name(condition.operand2.name) + ')'
           else:
             operand_value = '$(params.' + condition.operand2.name + ')'
           input_params.append(operand_value)
@@ -409,6 +416,14 @@ class TektonCompiler(Compiler):
         if condition_refs.get(parent_group[-2], []):
           task['conditions'] = condition_refs.get(op_name_to_parent_groups[task['name']][-2], [])
       if op.dependent_names:
+        for dependent_name in op.dependent_names:
+          if condition_refs.get(dependent_name, []):
+            # Prompt an error here because Tekton condition cannot be a dependency.
+            raise TypeError(textwrap.dedent("""\
+         '%s' cannot run after the Tekton condition '%s'.
+          A Tekton task is only allowed to run 'after' another Tekton task (ContainerOp).
+          Tekton doc: https://github.com/tektoncd/pipeline/blob/master/docs/pipelines.md#using-the-runafter-parameter
+          """ % (task['name'], dependent_name)))
         task['runAfter'] = op.dependent_names
 
     # process input parameters from upstream tasks
@@ -423,15 +438,14 @@ class TektonCompiler(Compiler):
             if tp['name'] == pp.full_name:
               tp['value'] = '$(tasks.%s.results.%s)' % (pp.op_name, pp.name)
               # Create input artifact tracking annotation
-              if self.enable_artifacts:
-                input_annotation = self.input_artifacts.get(task['name'], [])
-                input_annotation.append(
-                    {
-                        'name': tp['name'],
-                        'parent_task': pp.op_name
-                    }
-                )
-                self.input_artifacts[task['name']] = input_annotation
+              input_annotation = self.input_artifacts.get(task['name'], [])
+              input_annotation.append(
+                  {
+                      'name': tp['name'],
+                      'parent_task': pp.op_name
+                  }
+              )
+              self.input_artifacts[task['name']] = input_annotation
               break
 
     # add retries params
@@ -443,7 +457,7 @@ class TektonCompiler(Compiler):
     # add timeout params to task_refs, instead of task.
     for task in task_refs:
       op = pipeline.ops.get(task['name'])
-      if op.timeout:
+      if not TEKTON_GLOBAL_DEFAULT_TIMEOUT or op.timeout:
         task['timeout'] = '%ds' % op.timeout
 
     # handle resourceOp cases in pipeline
@@ -476,10 +490,14 @@ class TektonCompiler(Compiler):
       'kind': 'PipelineRun',
       'metadata': {
         'name': sanitize_k8s_name(pipeline.name or 'Pipeline', suffix_space=4),
-        'labels': get_default_telemetry_labels(),
+        # 'labels': get_default_telemetry_labels(),
         'annotations': {
           'tekton.dev/output_artifacts': json.dumps(self.output_artifacts, sort_keys=True),
           'tekton.dev/input_artifacts': json.dumps(self.input_artifacts, sort_keys=True),
+          'tekton.dev/artifact_bucket': DEFAULT_ARTIFACT_BUCKET,
+          'tekton.dev/artifact_endpoint': DEFAULT_ARTIFACT_ENDPOINT,
+          'tekton.dev/artifact_endpoint_scheme': DEFAULT_ARTIFACT_ENDPOINT_SCHEME,
+          'tekton.dev/artifact_items': json.dumps(self.artifact_items, sort_keys=True),
           'sidecar.istio.io/inject': 'false'  # disable Istio inject since Tekton cannot run with Istio sidecar
         }
       },
@@ -525,7 +543,7 @@ class TektonCompiler(Compiler):
       pipeline_run['spec']['taskRunSpecs'] = task_run_spec
 
     # add workflow level timeout to pipeline run
-    if pipeline.conf.timeout:
+    if not TEKTON_GLOBAL_DEFAULT_TIMEOUT or pipeline.conf.timeout:
       pipeline_run['spec']['timeout'] = '%ds' % pipeline.conf.timeout
 
     # generate the Tekton podTemplate for image pull secret
@@ -567,6 +585,8 @@ class TektonCompiler(Compiler):
           sanitized_attribute_outputs[sanitize_k8s_name(key, True)] = \
             op.attribute_outputs[key]
         op.attribute_outputs = sanitized_attribute_outputs
+      if isinstance(op, dsl.ContainerOp) and op.container is not None:
+        sanitize_k8s_object(op.container)
       sanitized_ops[sanitized_name] = op
     pipeline.ops = sanitized_ops
 
@@ -581,7 +601,6 @@ class TektonCompiler(Compiler):
                        pipeline_description: Text = None,
                        params_list: List[dsl.PipelineParam] = None,
                        pipeline_conf: dsl.PipelineConf = None,
-                       allow_telemetry: bool = True,
                        ) -> Dict[Text, Any]:
     """ Internal implementation of create_workflow."""
     params_list = params_list or []
@@ -646,12 +665,12 @@ class TektonCompiler(Compiler):
 
     op_transformers = [add_pod_env]
 
-    # By default adds telemetry instruments. Users can opt out toggling
-    # allow_telemetry.
-    # Also, TFX pipelines will be bypassed for pipeline compiled by tfx>0.21.4.
-    if allow_telemetry:
-      pod_labels = get_default_telemetry_labels()
-      op_transformers.append(add_pod_labels(pod_labels))
+    # # By default adds telemetry instruments. Users can opt out toggling
+    # # allow_telemetry.
+    # # Also, TFX pipelines will be bypassed for pipeline compiled by tfx>0.21.4.
+    # if allow_telemetry:
+    #   pod_labels = get_default_telemetry_labels()
+    #   op_transformers.append(add_pod_labels(pod_labels))
 
     op_transformers.extend(pipeline_conf.op_transformers)
 
@@ -684,9 +703,7 @@ class TektonCompiler(Compiler):
               pipeline_func,
               package_path,
               type_check=True,
-              pipeline_conf: dsl.PipelineConf = None,
-              allow_telemetry: bool = True,
-              enable_artifacts=True):
+              pipeline_conf: dsl.PipelineConf = None):
     """Compile the given pipeline function into workflow yaml.
     Args:
       pipeline_func: pipeline functions with @dsl.pipeline decorator.
@@ -695,11 +712,8 @@ class TektonCompiler(Compiler):
       pipeline_conf: PipelineConf instance. Can specify op transforms,
                      image pull secrets and other pipeline-level configuration options.
                      Overrides any configuration that may be set by the pipeline.
-      enable_artifacts: enable artifacts, requires Kubeflow Pipelines with Minio.
     """
-    self.enable_artifacts = enable_artifacts
-    super().compile(pipeline_func, package_path, type_check, pipeline_conf=pipeline_conf,
-                    allow_telemetry=allow_telemetry)
+    super().compile(pipeline_func, package_path, type_check, pipeline_conf=pipeline_conf)
 
   @staticmethod
   def _write_workflow(workflow: Dict[Text, Any],
@@ -774,7 +788,6 @@ class TektonCompiler(Compiler):
                                  params_list: List[dsl.PipelineParam] = None,
                                  pipeline_conf: dsl.PipelineConf = None,
                                  package_path: Text = None,
-                                 allow_telemetry: bool = True
                                  ) -> None:
     """Compile the given pipeline function and dump it to specified file format."""
     workflow = self._create_workflow(
@@ -782,8 +795,7 @@ class TektonCompiler(Compiler):
         pipeline_name,
         pipeline_description,
         params_list,
-        pipeline_conf,
-        allow_telemetry)
+        pipeline_conf)
     TektonCompiler._write_workflow(workflow=workflow, package_path=package_path)   # Tekton change
     _validate_workflow(workflow)
 
